/*******************************************************
* Zeus CMD - OpenGL ES Tutorial 24 : Uncompressed TGAs *
* By Grant James (ZEUS)                                *
* http://www.zeuscmd.com                               *
*******************************************************/
#pragma comment(lib, "libGLES_CM.lib")

#define GLUTES_STATIC

#include <GLES/glutes.h>
#include <GLES/glues.h>
#include <GLES/glues.c>

GLuint texture[2];

char *tgas[] = {
	"zeus.tga",
	"rockchecker.tga"
};

float bgVertices[] = {
	 1.0f, -1.0f,
	 1.0f,  1.0f,
	-1.0f, -1.0f,
	-1.0f,  1.0f
};

float bgTexCoords[] = {
	3.0f, 0.0f,
	3.0f, 3.0f,
	0.0f, 0.0f,
	0.0f, 3.0f
};

float checkerCoords[] = {
	3.0f, 0.0f,
	3.0f, 3.0f,
	0.0f, 0.0f,
	0.0f, 3.0f
};

float wrapValue = 0.0f;

struct TGAFile
{
	unsigned char typeCode;
	short int width;
	short int height;
	unsigned char bpp;
	GLenum format;
	unsigned char *imageData;
};

bool loadUncompressedTGA(char *filename, TGAFile *tgaFile)
{
	FILE *file;

	unsigned char tmpRGB;

	unsigned char ucNull;
	unsigned short usNull;

	TCHAR path[256];
	char fullPath[256];
	GetModuleFileName(NULL, path, 256);

	TCHAR *pos = wcsrchr(path, '\\');
	*(pos + 1) = '\0';
	
	wcstombs(fullPath, path, 256);
	
	strcat(fullPath, filename);

	file = fopen(fullPath,"rb");

	if (!file)
	{
		MessageBox(NULL, L"Can't Find TGA", L"Error", MB_OK);
		return false;
	}

	fread(&ucNull,sizeof(unsigned char),1,file);
	fread(&ucNull,sizeof(unsigned char),1,file);

	fread(&tgaFile->typeCode,sizeof(unsigned char),1,file);

	if (tgaFile->typeCode != 2 && tgaFile->typeCode != 3)
	{
		MessageBox(NULL, L"Incorrect texture type", L"Error", MB_OK);
		fclose(file);
		return false;
	}

	fread(&usNull,sizeof(unsigned short),1,file);
	fread(&usNull,sizeof(unsigned short),1,file);
	fread(&ucNull,sizeof(unsigned char),1,file);
	fread(&usNull,sizeof(unsigned short),1,file);
	fread(&usNull,sizeof(unsigned short),1,file);

	fread(&tgaFile->width, sizeof(unsigned short), 1, file);
	fread(&tgaFile->height, sizeof(unsigned short), 1, file);
	fread(&tgaFile->bpp, sizeof(unsigned char), 1, file);

	fread(&ucNull,sizeof(unsigned char),1,file);

	int colorMode = tgaFile->bpp / 8;

	if (colorMode == 3)
		tgaFile->format = GL_RGB;
	else
		tgaFile->format = GL_RGBA;

	long imageSize = tgaFile->width * tgaFile->height * colorMode;

	tgaFile->imageData = new unsigned char[imageSize];

	if (!tgaFile->imageData)
	{
		MessageBox(NULL, L"Out of Memory", L"Error", MB_OK);
		delete[] tgaFile->imageData;
		fclose(file);
		return false;
	}

	fread(tgaFile->imageData,sizeof(unsigned char),imageSize,file);

	if (!tgaFile->imageData)
	{
		MessageBox(NULL, L"Error reading TGA", L"Error", MB_OK);
		fclose(file);
		return false;
	}

	for (int i = 0; i < imageSize; i += colorMode)
	{
		tmpRGB = tgaFile->imageData[i];
		tgaFile->imageData[i] = tgaFile->imageData[i+2];
		tgaFile->imageData[i+2] = tmpRGB;
	}

	fclose(file);

	return true;
}

bool loadTextures()
{
	TGAFile tgaFile;

	glGenTextures(2, texture);

	for (int i = 0; i < 2; i++)
	{
		if (!loadUncompressedTGA(tgas[i], &tgaFile))
			return false;

		glBindTexture(GL_TEXTURE_2D, texture[i]);

		glTexImage2D(GL_TEXTURE_2D, 0, tgaFile.format, tgaFile.width,
			tgaFile.height, 0, tgaFile.format, GL_UNSIGNED_BYTE,
			tgaFile.imageData);

		glTexParameterf(GL_TEXTURE_2D, 
			GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D,
			GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		delete[] tgaFile.imageData;
	}

	return true;
}

bool init()
{
	if (!loadTextures())
	{
		MessageBox(NULL, L"Error loading textures", L"Error", MB_OK);
		return false;
	}

	glEnable(GL_TEXTURE_2D);

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepthf(1.0f);

	glEnable(GL_CULL_FACE);
	glShadeModel(GL_SMOOTH);

	glVertexPointer(2, GL_FLOAT, 0, bgVertices);
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	return true;
}

void display()
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	glTranslatef(0.0f, 0.0f, -2.0f);

	glTexCoordPointer(2, GL_FLOAT, 0, bgTexCoords);
	glBindTexture(GL_TEXTURE_2D, texture[0]);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

	glTexCoordPointer(2, GL_FLOAT, 0, checkerCoords);
	glBindTexture(GL_TEXTURE_2D, texture[1]);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

	glFlush ();
	glutSwapBuffers();
}

void idle()
{
	wrapValue += 0.1f;

	if (wrapValue > 1.0f)
		wrapValue = 0.0f;

	checkerCoords[0] = 3.0f + wrapValue;
	checkerCoords[2] = 3.0f + wrapValue;
	checkerCoords[4] = 0.0f + wrapValue;
	checkerCoords[6] = 0.0f + wrapValue;

	glutPostRedisplay();
}

void keyboard2(int key, int x, int y)
{
	switch(key)
	{
	case GLUT_KEY_UP : exit(0); break;
	}
}

void reshape(int width, int height)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glViewport(0, 0, width, height);
	gluPerspectivef(45.0f, 1.0f * width / height, 1.0f, 100.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void menu(int entry)
{
	switch(entry)
	{
	case 1 : exit(0); break;
	}
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);

	glutCreateWindow("24 - Uncompressed TGAs");

	if (!init())
		return 1;

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutSpecialFunc(keyboard2);

	glutIdleFunc(idle);

	glutCreateMenu(menu);
	glutAddMenuEntry("Quit", 1);

	glutAttachMenu(GLUT_LEFT_BUTTON);
	
	glutMainLoop();

	return 0;
}
